import 'package:flutter/material.dart';

/// Animation types
enum AnimationType { normal, fadeIn }

/// Main class, [context] It is the BuildContext of the application at that moment
/// [child] It is the Widget to navigate, [animation] It is the animation type
class NavigatorTransitionsRoute {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  NavigatorTransitionsRoute(
      {@required this.context,
      @required this.child,
      this.replacement = false,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 200)}) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  /// Push normal navigator of page
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Push replacement navigator of page
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  /// Controller of normal transition
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Controller of fade in transition
  void _fadeInTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
              child: child,
              opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                  CurvedAnimation(parent: animation, curve: Curves.easeOut)));
        });

    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }
}
